@@ -1,6 +1,6 @@
 use super::any::PyAnyMethods;
-
-use crate::{ffi, instance::Bound, PyAny};
+use crate::{ffi, instance::Bound, IntoPyObject, PyAny, Python};
+use std::convert::Infallible;
 
 /// Represents a Python `int` object.
 ///
@@ -20,6 +20,21 @@ pyobject_native_type_core!(PyInt, pyobject_native_static_type_object!(ffi::PyLon
 #[deprecated(since = "0.23.0", note = "use `PyInt` instead")]
 pub type PyLong = PyInt;
 
+impl PyInt {
+    /// Creates a new Python int object.
+    ///
+    /// Panics if out of memory.
+    pub fn new<'a, T>(py: Python<'a>, i: T) -> Bound<'a, PyInt>
+    where
+        T: IntoPyObject<'a, Target = PyInt, Output = Bound<'a, PyInt>, Error = Infallible>,
+    {
+        match T::into_pyobject(i, py) {
+            Ok(v) => v,
+            Err(never) => match never {},
+        }
+    }
+}
+
 macro_rules! int_compare {
     ($rust_type: ty) => {
         impl PartialEq<$rust_type> for Bound<'_, PyInt> {
@@ -60,6 +75,7 @@ int_compare!(usize);
 
 #[cfg(test)]
 mod tests {
+    use super::*;
     use crate::{IntoPyObject, Python};
 
     #[test]
@@ -123,4 +139,18 @@ mod tests {
             }
         });
     }
+
+    #[test]
+    fn test_display_int() {
+        Python::with_gil(|py| {
+            let s = PyInt::new(py, 42u8);
+            assert_eq!(format!("{}", s), "42");
+
+            let s = PyInt::new(py, 43i32);
+            assert_eq!(format!("{}", s), "43");
+
+            let s = PyInt::new(py, 44usize);
+            assert_eq!(format!("{}", s), "44");
+        })
+    }
 }
